//  MUWidget.h version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//
//

#ifndef __muwidget_h__
#define __muwidget_h__

#include <UWidget.h>
#include "MUXKinput.h"
#include <AString.h>
#include "MUColor.h"

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>

class ScrollValue
{
public:
			ScrollValue (void);
			~ScrollValue ();
	int		value;
	int		max;
	int		step;
	int		page;
};

class MUWidget : public UWidget
{
public:
				MUWidget (const char* nameIn, Widget parent, 
					const UEditable editableIn=EDITABLE);

				~MUWidget ();
	void			setupWidget (const char* fontIn,
					int		fontSizeIn,
					const char* 	fontWeightIn,
					const char*	fontSlantIn,
					const char*	fontSpacingIn,
					const char*	fontAddStyleIn,
					const char*	fontAvgWidthIn
					);
	void			setupWidget (UWidget* uwidgetIn);

	inline void		clearText () { uClear(); }
	void			appendText (const unsigned char* text, const int len);
	unsigned char*		getLine (const int pos, int* len);
	unsigned char*		getBuffer(int *len);
	void			refresh();

	UString::UStatus 	setConversion (const char *conversion);
	UWidget::UStatus 	setInput (const char *input);
	void			setTabSize (int newValue);

	void			setBackground (const char* color);
	void			setForeground (const char* color);
	void			setCursorColor (const char* color);
	unsigned char*		clipBoard;
	int			clipBoardLen;
	inline Widget		getWidget () { return editArea; }

	void			setMultiLine (UWidget::UMultiLine multiLineIn);

	int			uSetXInput (const char*	name,
					const char*	imIn,
					const char*	inputStyleIn,
					const char*	textType,
					const char*	uStringIn,
					const char*	fontIn,
					double		verisonIn=0.0);
protected:
	enum XKinputType	{ KinputType, NoneType };
	UXInput* 		xInput;
	UFontX11*		xInputFont;
	UFontMap*		xInputFontMap;
	AString			xInputFontMapName;
	XKinputType		xInputType;

	char*			pasteString;
	int			pasteStringLen;
	int			pasteStarted;

	MUColor			background;
	MUColor			foreground;
	MUColor			cursorColor;
	//
	// overriden methods from UWidget 
	//
	virtual void 		uClipboardPut (const unsigned char *, const int len);
	virtual unsigned char*	uClipboardGet (int *len);
	virtual void		uSetVScroll (int value, int step, int page, int
				max);
	virtual void		uSetHScroll (int value, int step, int page, int
				max);

	Widget			parent;
	Widget			hbar;
	ScrollValue		hValues;
	Widget			vbar;
	ScrollValue		vValues;
	Widget			editArea;
	Widget			editFrame;
	int			timerOn;
	XtIntervalId		timerId;
	friend void 		MuedHScolled (Widget sb, MUWidget* uwidget,
					XmScrollBarCallbackStruct *cbs);
	friend void 		MuedVScolled (Widget sb, MUWidget* uwidget,
					XmScrollBarCallbackStruct *cbs);
	friend void 		MuedResized (Widget sb, MUWidget* uwidget,
					XmAnyCallbackStruct *cbs);
	friend void 		MuedEvent (Widget sb, MUWidget* uwidget,
					XmDrawingAreaCallbackStruct *cbs);
	friend void 		MuedFocusIn (Widget sb, MUWidget* uwidget,
					XmAnyCallbackStruct *cbs);
	friend void 		MuedFocusOut (Widget sb, MUWidget* uwidget,
					XmAnyCallbackStruct *cbs);
	friend void 		MuedTimerCB (MUWidget *muwidget, XtIntervalId *id);
	friend void		MUEditLoseSelection (Widget w, Atom* selection);

	friend void 		MUEditGetSelection (Widget w, MUWidget* uwidget,
					Atom* selection, Atom* type, 
					XtPointer value, unsigned long *length,
					int* format);

	friend void 		MUEditEventHandler (Widget w, 
				 MUWidget* muwidget, XEvent* event, 
				 Boolean* callAgain);
};


#endif /* __muwidget_h__ */
