//  MainWidget.h version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
#ifndef __main_widget_h__
#define __main_widget_h__

#include <Xm/Xm.h>
#include <Config.h>
#include "Dialog.h"
#include "FileDialog.h"
#include <MUWidget.h>
#include <MUXKinput.h>
#include <UFontFreeType.h>
#include "PrintDialog.h"
#include "FindDialog.h"

class MenuBarAction
{
public:
			MenuBarAction (const char* name);
	virtual		~MenuBarAction ();
	enum MenuId	{UFILE, UENCODING, UINPUT, UXINPUT, UFONTNAME, 
				UFONT_ADD_STYLE, UFONT_AVG_WIDTH,
				UFONTWEIGHT, UFONTSIZE, UFONTSLANT,
				UFONTSPACING, UTABSIZE, UWINDOW_BACKGROUND,
				UWINDOW_FOREGROUND, UCURSOR_COLOR, 
				USEARCH, UHELP, UMENUMAX};
	enum MenuState  {UON, UOFF};
	enum MenuType	{NORMAL, RADIO, TOGGLE};
	virtual	void	clickedOn (const MenuId menuIn, 
				const char* stringIn, const MenuState onOff);
};

class MenuRC
{
public:			
	enum ButtonType	{SEPARATOR, BUTTON, TYPE_MAX};
			MenuRC (Widget			parentWidget,
				MenuBarAction* 		parentIn, 
				MenuBarAction::MenuId	idIn,
				MenuBarAction::MenuType	typeIn, 
				const char*             nameTop,
				const char*		namePulldown);
			~MenuRC ();
	inline MenuBarAction*		getParent () { return parent; }
	inline MenuBarAction::MenuId 	getId () { return id; }
	void		addItem (ButtonType buttonTypeIn, const char *nameIn);
	void		addButtons (const AStringList&  namesIn);
	void 		clickOn (const char* stringIn, int callBack=0);
	Widget			manager;
	Widget			menuButton;
private:
	void		translate (const char* nameIn, char** pureOrig, 
				char** menuName, char** menuMneno);

	friend void		MButtonCB (Widget w, MenuRC* mainWidget, 
					XmAnyCallbackStruct *cb);
	MenuBarAction::MenuId	id;
	MenuBarAction::MenuType	type;
	MenuBarAction*		parent;

	Widget			menuPopup;
	Widget*			menuItems;

	int			menuSize;
	int			menuArraySize;
};

class MainWidgetAction
{
public:
			MainWidgetAction (const Config& configIn);
	virtual		~MainWidgetAction ();
	virtual	void	clickedOn (const MenuBarAction::MenuId menuIn, 
				const char* stringIn, const MenuBarAction::MenuState onOff);
	Widget		mainForm;
	inline const Config&	getConfig() const { return config; }
protected:
	Config		config;
};

class MenuBar : public MenuBarAction
{
public:
				MenuBar (MainWidgetAction* mainWidgetIn,
					const char* name);
	virtual			~MenuBar ();
	Widget			menuBar;
	void 			clickOn (const MenuId menuIn, 
					const char* stringIn, int callBack=0);
private:
	MainWidgetAction*	mainWidget;
	MenuRC*			menu[UMENUMAX];
	MenuRC*			menuWindow;

	virtual	void		clickedOn (const MenuId menuIn, const char* stringIn, const MenuBarAction::MenuState onOff);
};

class MainWidget : public MainWidgetAction
{
public:
	enum YesNo	{YES, NO};
			MainWidget (
				const char* name, 
				Widget parent,
				const Config& configIn, 
				const char* fileNameIn=0,
				const char* encodingIn=0,
				const UWidget::UEditable=UWidget::EDITABLE);
	virtual		~MainWidget ();
	Widget		getParent() {return parent;}
	const char*	getName() {return name;}
	void		warn (const char* messageIn);
	void		inform (const char* messageIn);
	YesNo		ask (const char* messageIn);
private:

	UWidget::UEditable	isEditable;
	int			setXInput (const char* name);
	AString			fileName;
	AString			encoding;
	Widget			parent;
	const char*		name;
	Widget			mainWidget;
	MenuBar*		menuBar;
	Widget			editForm;
	MUWidget*		editArea;
	FileDialog*		saveDialog;
	FileDialog*		loadDialog;
	PrintDialog*		printDialog;
	FindDialog*		findDialog;
	int			checkEncoding();
	int 			saveFile (const char* filenameIn);
	int 			loadFile (const char* filenameIn);
	int 			appendFile (const char* filenameIn);
	int			appendFileBase (const char* filenameIn);

	friend void 	MDestroyMain (Widget w, 
				MainWidget* mainWidget, XtPointer p);
	void		setToggle ();
	friend Boolean	MSetToggleCB (MainWidget* mainWidget);

	virtual	void	clickedOn (const MenuBarAction::MenuId menuIn, 
				const char* stringIn, const MenuBarAction::MenuState onOff);
	int		getInstances ();
	MainWidget*	getFirst ();
};


#endif /* __main_widget_h__ */
